package com.wb.excel.api.enumeration;

import com.wb.excel.api.annotation.ColumnDescription;
import com.wb.excel.api.annotation.ColumnName;
import com.wb.excel.api.datatable.WCell;
import com.wb.excel.api.util.EnumUtil;
import com.wb.excel.api.util.StringUtil;
import com.wb.excel.api.util.ValidationUtil;

/**
 * 数据类型枚举类。包含了目前DataTable支持验证的11种类型。<br/>
 * DataTable会对指定的类型进行格式上的验证。
 * Created on 2014/9/19/.
 *
 * @author
 * @version v1.0.0.0
 */
public enum DataType {
    @ColumnName("字符型")
    @ColumnDescription("普通的字符串类型，例如：abc123.,!@#")
    STRING,
    @ColumnName("整数型")
    @ColumnDescription("整数类型，小数点后的数字会被抹去（不是四舍五入）,例如：11，200，4000")
    NUMBER,
    @ColumnName("数字类型")
    @ColumnDescription("可以带小数点的数字类型，例如：1.00，1.01")
    DECIMAL,
    @ColumnName("日期型")
    @ColumnDescription("普通的日期类型，例如：2014-10-01")
    DATE,
    @ColumnName("时间日期型（秒）")
    @ColumnDescription("时间精确到秒的日期类型，例如：2014-10-01 10:30:00")
    DATETIME,
    @ColumnName("是否型")
    @ColumnDescription("指定是或者否的类型，例如：是，否")
    BOOLEAN;

    /**
     * 匹配格式.
     *
     * @param type  要判断的类型
     * @param WCell 需要判断类型的单元格（可能会有改变值的情况发生，所以需要传入此值）
     * @param value 已被简单处理过的需要判断的值
     * @return 匹配结果。true，符合；false，不符合
     */
    public static boolean check(DataType type, WCell WCell, String value) {
        boolean typeFlag = true;
        if (value.length() > 0) {
            switch (type) {
                case DECIMAL:
                    typeFlag = ValidationUtil.checkDouble(value);
                    break;
                case NUMBER:
                    typeFlag = ValidationUtil.checkInteger(value);
                    if (typeFlag) {
                        WCell.setValue(StringUtil.transferInteger(value));
                    }
                    break;
                case DATE:
                    typeFlag = ValidationUtil.checkDate(value);
                    if (typeFlag) {
                        WCell.setValue(StringUtil.transferDate(value));
                    }
                    break;
                case DATETIME:
                    typeFlag = ValidationUtil.checkDatetime(value);
                    if (typeFlag) {
                        WCell.setValue(StringUtil.transferDatetime(value));
                    }
                    break;
                case BOOLEAN:
                    typeFlag = EnumUtil.check(YesNo.class, value);
                    if (typeFlag) {
                        WCell.setValue(StringUtil.transferBoolean(value));
                    }
                    break;
                default:
                    break;
            }
        }
        return typeFlag;
    }
}
